/******************************************************************************
    CallWave VoiceMail Widget

    Copyright (C) 2006 CallWave, Inc.  All Rights Reserved.

 ******************************************************************************/

/******************************************************************************
  Constants
 ******************************************************************************/

var VM_RESOURCE_SELECTED_ROW_IMAGE    = "Images/y_selectedrow_background.png";

var VM_RESOURCE_PHONE_ICON_HOME       = "Images/y_icon_home.png";
var VM_RESOURCE_PHONE_ICON_WORK       = "Images/y_icon_work.png";
var VM_RESOURCE_PHONE_ICON_CELL       = "Images/y_icon_cell.png";
var VM_RESOURCE_PHONE_ICON_OTHER      = "Images/y_icon_other.png";

var VM_RESOURCE_PHONE_ICON_HOME_W     = "Images/y_icon_home_w.png";
var VM_RESOURCE_PHONE_ICON_WORK_W     = "Images/y_icon_work_w.png";
var VM_RESOURCE_PHONE_ICON_CELL_W     = "Images/y_icon_cell_w.png";
var VM_RESOURCE_PHONE_ICON_OTHER_W    = "Images/y_icon_other.png";

var VM_RESOURCE_PLAY_IMAGE            = "Images/y_play.png";
var VM_RESOURCE_EDIT_IMAGE            = "Images/y_edit.gif";
var VM_RESOURCE_DELETE_IMAGE          = "Images/y_delete.png";
var VM_RESOURCE_STOP_IMAGE            = "Images/y_stop.png";

var VM_RESOURCE_PLAYER_STUB_IMG     = { src: "Images/a_player_stub.png", height: 26, width: 142, alt: "[----|---------]" };

var VM_RESOURCE_MWI_SOUND             = "Resources/MWI.wav";

var VM_CARRIER_ID_UNDEFINED           = -1;
var VM_CARRIER_ID_OTHER               = 0;

/******************************************************************************
  VM_CallListView class
 ******************************************************************************/

/******************************************************************************
  VM_CallListView C'tor

    Exceptions:
        None

 ******************************************************************************/
function VM_CallListView( callListLength, initialShowActivationPreference, onLoginRequired )
{
    // User preferences
    this.m_maxCalls                 = callListLength;
    this.m_showActivationPreference = initialShowActivationPreference;

    // Edit variables
    this.m_editingCallId             = null;

    // View state
    this.m_selectedRowIndexOf       = -1;   // index in calllist of the 'selected' or hot call
    this.m_selectedCallId           = null; //  CL_CallId of the 'selected' or hot call

    this.m_callList                 = new CL_CallList( );
    this.m_viewHasBeenShownForCurrentAccount = false;

    // Timers
    this.m_syncTimer                = null;

    // Authentication response attributes
    this.m_secondsBetweenSync   = this.SYNC_INTERVAL_SECONDS;
    this.m_activationStatus     = null;
    this.m_carrierId            = VM_CARRIER_ID_OTHER;
    this.m_forwardCodeList      = null;
    this.m_carrierList          = null;
    this.m_hideRegCarrier       = 0;

    // Alert pane management
    this.m_alertMessageTimeStamp         = 0;
    this.m_gettingCallsAlertMsgTimeStamp = 0;
    this.m_carrierInfoAlertMsgTimeStamp  = 0;
    this.m_undeleteAlertMsgTimestamp     = 0;


    this.m_onLoginRequired = onLoginRequired;

    // Call list sync
    var self = this;

    var getAccountNumberCallBack = function( )
    {
        return VM_GetPreference( PREF_KEY_ACCOUNT );
    };
    var getUStringCallBack = function( )
    {
        return VM_GetPreference( PREF_KEY_U_STRING );
    };
    var getMaxCallsCallBack = function( )
    {
        return self.m_maxCalls;
    };

    this.m_callListHandler = new CLH_CallListHandler( VM_WIDGET_SOURCE,
                                                      VM_WIDGET_VERSION,
                                                      VM_WIDGET_OS_VERSION,
                                                      getAccountNumberCallBack,
                                                      getUStringCallBack,
                                                      getMaxCallsCallBack );

    this.m_callListHandler.m_onCallListChanged   = function( statusMessage )
    {
        self.OnCallListChanged( statusMessage );
    };
    this.m_callListHandler.m_onCallListChangedAndActivated   = function( statusMessage )
    {
        self.OnCallListChangedAndActivated( statusMessage );
    };
    this.m_callListHandler.m_onSyncFailed        = function( errorMessage )
    {
        self.OnSyncFailed( errorMessage );
    };
    this.m_callListHandler.m_onCallListSyncCompletedNoChange = function( statusMessage )
    {
        self.OnCallListSyncCompletedNoChange( statusMessage );
    };
    this.m_callListHandler.m_onNewCallsAvailable = function( )
    {
        self.OnNewCallsAvailable( );
    };
    this.m_callListHandler.m_onLoginRequired     = onLoginRequired;

    // Activation

    // set up some closures
    var onSetCarrierInfoSucceeded = function( responseMessage )
    {
        self.OnSetCarrierInfoSucceeded( responseMessage );
    };

    var onSetCarrierInfoFailed = function( statusMessage )
    {
        self.OnSetCarrierInfoFailed( statusMessage );
    };

    this.m_activationHandler = new ACT_ActivationHandler( VM_WIDGET_SOURCE,
                                                          VM_WIDGET_VERSION,
                                                          VM_WIDGET_OS_VERSION,
                                                          onSetCarrierInfoSucceeded,
                                                          onSetCarrierInfoFailed );
}
VM_CallListView.prototype =
{
    /**************************************************************************
      Constants
     **************************************************************************/
    SYNC_INTERVAL_SECONDS       : 20,
    MINIMAL_SYNC_MULTIPLIER     : 10,
    MAX_ALERT_MESSAGE_LENGTH    : 60,

    MAX_CALLER_NAME_COLUMN_PIXEL_WIDTH : 126,
    MAX_DATE_COLUMN_PIXEL_WIDTH        : 48,
    MAX_TIME_COLUMN_PIXEL_WIDTH        : 58,
    MAX_DURATION_COLUMN_PIXEL_WIDTH    : 64,

    VTEXT_DISPLAY_DELAY_MS  : 200,


    /**************************************************************************
      Accessor, modifier methods
     **************************************************************************/

    IsCallReviewActive:function( )
    {
        return ($(".playing", "#calls").length > 0);
    },

    isEditing:function( )
    {
        return this.m_editingCallId !== null;
    },

    cancelEditing: function( )
    {
        try
        {
            if ( ! this.isEditing( ) )
            {
                return;
            }

            var call = $( "#" + this.m_editingCallId );

            this.ShowInfoHideEditPortions( call );

            VM_CallListView_static.EnableVtxt( );

            this.m_editingCallId = null;
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "VM_CallListView::cancelEditing( )" );
        }
    },

    /**************************************************************************
      View reset methods
     **************************************************************************/

    Hide:function( )
    {
        // hide the frame
        document.getElementById( 'WIDGET_callListViewFrame' ).style.display = 'none';
    },

    Show:function( )
    {
        // clear status areas
        document.getElementById( 'WIDGET_activationStatusTextArea' ).innerHTML = '';

        // only clear the alert area if the call list view has been shown before.
        if ( this.m_viewHasBeenShownForCurrentAccount )
        {
            this.ClearAlert( );
        }

        // Hide objects
        document.getElementById( 'WIDGET_callListViewFrame' ).style.display = 'block';

        this.m_viewHasBeenShownForCurrentAccount = true;
    },

    ResetAccountInfo : function( )
    {
        // reset the carrier info
        this.m_carrierId        = VM_CARRIER_ID_OTHER;
        this.m_forwardCodeList  = null;
        this.m_carrierList      = null;
        this.m_hideRegCarrier   = 0;

        this.m_callListHandler.clearCallList( );
        this.m_viewHasBeenShownForCurrentAccount = false;

        // reset the activation status
        this.m_activationStatus = null;
    },

    ShouldShowActivation:function( )
    {
        return ( ( this.m_activationStatus != null &&
                   this.m_activationStatus == TYPE_ACTIVATION_STATE_NOT_ACTIVATED )
                 || this.m_showActivationPreference );
    },

    UpdatePreferences : function( maxCalls, showActivationPreference )
    {
        if (this.m_maxCalls == maxCalls && this.m_showActivationPreference == showActivationPreference) {
            return false;
        }
        this.m_maxCalls = maxCalls;
        this.m_showActivationPreference = showActivationPreference;
        return true;
    },

    OnLoginSucceeded : function( secondsBetweenSync,
                                 activationStatus,
                                 carrierId,
                                 forwardCodeList,
                                 carrierList,
                                 hideRegCarrier )
    {
        try
        {
            //alert( "CallListView.OnLoginSucceeded( secondsBetweenSync:" + secondsBetweenSync +
            //         ", activationStatus:" + activationStatus + ", carrierId:" + carrierId + ", " + forwardCodeList + ", " + carrierList + ")");
            // copy over the attributes
            this.m_secondsBetweenSync   = ( secondsBetweenSync == 0 ? this.SYNC_INTERVAL_SECONDS : secondsBetweenSync );
/* DEBUG LEAK */
/* this.m_secondsBetweenSync = 1; /* spin pretty much as fast as we can */
            this.m_activationStatus     = activationStatus;
            this.m_carrierId            = carrierId;
            this.m_forwardCodeList      = forwardCodeList;
            this.m_carrierList          = carrierList;
            this.m_hideRegCarrier       = hideRegCarrier;


            // replace newlines in the forward codes with HTML breaks
            for ( var i = 0; i < this.m_forwardCodeList.m_forwardCodeList.length; ++i )
            {
                var forwardCode = this.m_forwardCodeList.m_forwardCodeList[ i ];
                forwardCode.m_actStr   = forwardCode.m_actStr.replace( /\n/g, '<br />' );
                forwardCode.m_deactStr = forwardCode.m_deactStr.replace( /\n/g, '<br />' );
            }

            if ( this.m_hideRegCarrier === 0 )
            {
                this.RefreshWirelessProviderPopup( );
            }
            else
            {
                this.SetAndDisplayWirelessProviderInfo( );
            }

            this.m_gettingCallsAlertMsgTimeStamp = this.showAlert( "Getting calls..." );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::OnLoginSucceeded( )' );
        }
    },

    RefreshWirelessProviderPopup : function( )
    {
        try
        {
            function AddOption( popup, option )
            {
                try
                {
                    popup.add( option, null );
                }
                catch ( e )
                {
                    popup.add( option );
                }
            }

            if ( this.m_carrierList            == null ||
                 this.m_carrierList.m_carriers == null )
            {
                return;
            }

            var bar = "-------------------";

            var carrierList       = this.m_carrierList.m_carriers;
            var selectedCarrierId = this.m_carrierId;

            var wirelessProviderPopup = document.getElementById( 'WIDGET_wirelessProviderPopup' );

            // remove all the providers
            while ( wirelessProviderPopup.length != 0 )
            {
                wirelessProviderPopup.remove( 0 );
            }

            var option = new Option( "Choose One", VM_CARRIER_ID_UNDEFINED );
            option.disabled = true;
            AddOption( wirelessProviderPopup, option );

            option = new Option( bar, VM_CARRIER_ID_UNDEFINED );
            option.disabled = true;
            AddOption( wirelessProviderPopup, option );

            // Add short-list first
            var shortListItemCount = 0;

            for ( var i = 0; i < carrierList.length; ++i )
            {
                if ( carrierList[i].m_shortlist == null ||
                     ! carrierList[i].m_shortlist )
                {
                    continue;
                }

                option = new Option( carrierList[i].m_name, carrierList[i].m_id );
                option.selected = carrierList[i].m_id == selectedCarrierId;
                AddOption( wirelessProviderPopup, option );

                ++shortListItemCount;
            }

            if ( shortListItemCount != 0 )
            {
                option = new Option( bar, VM_CARRIER_ID_UNDEFINED )
                option.disabled = true;
                AddOption( wirelessProviderPopup, option );
            }

            // Now add complete list

            for ( var i = 0; i < carrierList.length; ++i )
            {
                if ( carrierList[i].m_name == "Other" )
                {
                    // Add a separator before the 'Other' selection
                    option = new Option( bar, VM_CARRIER_ID_UNDEFINED )
                    option.disabled = true;
                    AddOption( wirelessProviderPopup, option );
                }

                option = new Option( carrierList[i].m_name, carrierList[i].m_id );
                option.selected = carrierList[i].m_id == selectedCarrierId;
                AddOption( wirelessProviderPopup, option );
            }
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::RefreshWirelessProviderPopup( )' );
        }
    },

    OnWirelessProviderSelected : function( )
    {
        try
        {
            var wirelessProviderPopup = document.getElementById( 'WIDGET_wirelessProviderPopup' );

            if ( this.m_carrierList            == null ||
                 this.m_carrierList.m_carriers == null )
            {
                wirelessProviderPopup.options[ 0 ].selected = true;
                return;
            }

            var providerId = wirelessProviderPopup.options[ wirelessProviderPopup.selectedIndex ].value;
            if ( providerId == VM_CARRIER_ID_UNDEFINED )
            {
                wirelessProviderPopup.options[ 0 ].selected = true;
                return;
            }

            // verify that the selected carrier is still available for selection
            var carrierList = this.m_carrierList.m_carriers;
            var carrierExists = false;
            for ( var i = 0; i < carrierList.length; ++i )
            {
                if ( carrierList[i].m_id == providerId )
                {
                    carrierExists = true;
                    break;
                }
            }

            if ( ! carrierExists )
            {
                wirelessProviderPopup.options[ 0 ].selected = true;
                return;
            }

            this.m_carrierId = providerId;

            this.SetAndDisplayWirelessProviderInfo( );

            this.SendCarrierInfo( );
        }
        catch ( e )
        {
            EX_Log( "VM_CallListView::OnWirelessProviderSelected( ) : " + e.message );
        }
    },

    /**************************************************************************
      View refresh methods
     **************************************************************************/

    refreshActivationFrame:function( )
    {
        try
        {
            if ( ! this.ShouldShowActivation( ) )
            {
                document.getElementById( 'WIDGET_activationFrame' ).style.display = 'none';

                return;
            }

            this.SetAndDisplayWirelessProviderInfo( );

            document.getElementById( 'WIDGET_activationFrame' ).style.display = 'block';
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::refreshActivationFrame( )' );
        }
    },

    refreshAlertFrame:function( )
    {
        try
        {
            document.getElementById( 'WIDGET_alertFrame' ).style.display = 'block';
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::refreshAlertFrame( )' );
        }
    },

    OnResize : function( )
    {
        // TBD - Could do something more minimal here...
        this.refresh( );
    },

    GetClientWidth: function( )
    {
        if ( self.innerWidth )
        {
            return self.innerWidth;
        }

        if ( document.documentElement && document.documentElement.clientWidth )
        {
            return document.documentElement.clientWidth;
        }

        if ( document.body )
        {
            return document.body.clientWidth;
        }

        return 436;
    },

    GetCallWidths: function( )
    {

        var availableCallWidth = this.GetClientWidth( );

        var callWidths = new Object( );

        callWidths.actionButton = Math.floor( availableCallWidth * .07 );
        callWidths.type         = Math.floor( availableCallWidth * .04 );
        callWidths.date         = Math.floor( availableCallWidth * .11 );
        callWidths.time         = Math.floor( availableCallWidth * .13 );
        callWidths.duration     = Math.floor( availableCallWidth * .12 );
        callWidths.editButton   = Math.floor( availableCallWidth * .07 );
        callWidths.deleteButton = Math.floor( availableCallWidth * .07 );

        // name gets the remainders
        callWidths.name         =
                availableCallWidth      - callWidths.actionButton -
                callWidths.type         - callWidths.date         -
                callWidths.time         - callWidths.duration     -
                callWidths.editButton   - callWidths.deleteButton;

        return callWidths;
    },

    refreshCallListFrame: function( )
    {
        try
        {
            var selectedIndex = this.m_selectedRowIndexOf;
            var selectedCallId = this.m_selectedCallId;

            var callListFrame = document.getElementById( 'WIDGET_callListFrame' );

            var callsShown = 0;

            var maxCalls = this.m_maxCalls;

            var callWidths = this.GetCallWidths( );

            var callListHTML = '';

            this.callRowStripe = '';

            // Display call list in the view (assume calls are in ascending order of call age)
            for ( var callListIndex = 0; callListIndex < this.m_callList.length( ) && callsShown < maxCalls; ++callListIndex )
            {
                // Get the next call
                var callListItem = this.m_callList.at( callListIndex );
                if ( callListItem.getReviewStatus( ) !== CL_REVIEW_STATE_NEW &&
                     callListItem.getReviewStatus( ) !== CL_REVIEW_STATE_SAVED )
                {
                    // Don't show this call
                    continue;
                }
                callListItem.selected = selectedCallId && callListItem.getCallId() == selectedCallId ? ' selected' : null;
                callListHTML += this.getCallRow( callListItem, callWidths );

                ++callsShown; // (always <= call list index)
            }

            callListFrame.innerHTML = tag('table', { id: 'calls', 'class': 'calls', cellspacing: 0, cellpadding: 0 }, callListHTML );

            // Show the call list frame
            callListFrame.style.display = 'block';
            if (window._IG_AdjustIFrameHeight) window._IG_AdjustIFrameHeight();
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::refreshCallListFrame( )' );
        }
    },

    refresh: function( )
    {
        //return; // refresh - return
        try
        {
            //return; // try { return
            // Draw activation frame
            this.refreshActivationFrame( );
            //alert( "refreshActivationFrame - DONE" );
            // Draw alert frame
            this.refreshAlertFrame( );
            //alert( "refreshAlertFrame - DONE" );

            // Draw calls frame
            this.refreshCallListFrame( );
            //alert( "refreshCallListFrame - DONE" );

            var calls = $("#calls");

            this.HideCallPortion( ".editInfo", calls );
            this.HideCallPortion( ".extra", calls );
            this.ShowCallPortion( ".info", calls );

            $(".call", calls).each( VM_CallListView_static.setupSingleCall );

            VM_CallListView_static.stripeIt( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::refresh( )' );
        }
    },


    /**************************************************************************
      Alert methods
     **************************************************************************/
    showAlert: function( messageText, _actionText )
    // takes messageText, [actionText], [deleteMessageAfterNSeconds]
    {
        try
        {
            var displayedMessageText =
                messageText.length > this.MAX_ALERT_MESSAGE_LENGTH ?
                    messageText.substr( 0, this.MAX_ALERT_MESSAGE_LENGTH - 3 ) + "..." :
                    messageText;
            var actionText = _actionText, secondsBeforeClearingMsg = null;
            if (arguments.length == 3) {
                secondsBeforeClearingMsg = arguments[2];
            }
            else if (arguments.length == 2 && typeof _actionText == 'number') {
                secondsBeforeClearingMsg = _actionText;
                actionText = null;
            }

            var alertMessage = document.getElementById( 'WIDGET_alertMessageTextArea' );

            if ( actionText == null || actionText.length == 0  )
            {
                alertMessage.innerHTML = displayedMessageText;
            }
            else
            {
                alertMessage.innerHTML = '<a href="#" onClick="' + actionText + '">' + displayedMessageText + '</a>';
            }

            alertMessage.alt       = messageText;

            var theDate = new Date( );
            var timeStamp = theDate.getTime( );
            this.m_alertMessageTimeStamp = timeStamp;

            if (secondsBeforeClearingMsg && secondsBeforeClearingMsg >= 5) { // wait at least 5 seconds before trying to clear a message
                var self = this;
                var thisStamp = timeStamp.toString();
                setTimeout( function() { self.ClearAlertWithTimeStamp( thisStamp ); }, secondsBeforeClearingMsg * 1000 );
            }

            return timeStamp;
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::showAlert( )' );
        }
    },

    ClearAlertWithTimeStamp : function( alertMessageTimeStamp )
    {
        try
        {
            if ( alertMessageTimeStamp == this.m_alertMessageTimeStamp )
            {
                this.ClearAlert( );
            }
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::ClearAlertWithTimeStamp( )' );
        }
    },

    ClearAlert : function( )
    {
        try
        {
            var alertMessage = document.getElementById( 'WIDGET_alertMessageTextArea' );
            alertMessage.innerHTML = '';

            this.m_alertMessageTimeStamp = 0;
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::ClearAlert( )' );
        }
    },


    /**************************************************************************
      Call List synchronization methods
     **************************************************************************/
    StartSynchronization: function( )
    {
        try
        {
            // cancel the existing interval
            this.DisableSynchronization( );

            // Perform initial synchronization
            this.OnSyncRequired( );

            // schedule subsequent synchronizations
            var self = this;
            var onSyncRequiredTimerCode = function( )
            {
                self.OnSyncRequired( );
            };

            this.m_syncTimer = setInterval( onSyncRequiredTimerCode, this.m_secondsBetweenSync * 1000 );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::StartSynchronization( )' );
        }
    },

    StartMinimalSynchronization : function( )
    {
        try
        {
            // cancel the existing interval
            this.DisableSynchronization( );

            // schedule subsequent synchronizations
            var self = this;
            var onSyncRequiredTimerCode = function( )
            {
                self.OnSyncRequired( );
            };

            this.m_syncTimer = setInterval( onSyncRequiredTimerCode, this.m_secondsBetweenSync * 1000 * this.MINIMAL_SYNC_MULTIPLIER );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::StartMinimalSynchronization( )' );
        }
    },

    DisableSynchronization: function( )
    {
        try
        {
            clearInterval( this.m_syncTimer );
            this.m_syncTimer = null;
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::DisableSynchronization( )' );
        }
    },

    OnSyncRequired: function( )
    {
        try
        {
            // Avoid synchronizing during message playback or editing.
            if ( this.IsCallReviewActive( ) || this.isEditing( ) || this.m_callListHandler.isRequestInProgress( ) )
            {
                return;
            }

            this.m_callListHandler.synchronize( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::OnSyncRequired( )' );
        }
    },

    OnCallListChanged: function( statusMessage )
    {
        try
        {
            this.m_callList = this.m_callListHandler.getCallList( );

            // Don't update the call list during message playback or editing. It will refresh
            // automatically once playback is complete.
            if ( this.IsCallReviewActive( ) || this.isEditing( ) )
            {
                return;
            }

            if ( statusMessage && statusMessage.length != 0 )
            {
                var url = statusMessage.match( "www[/./?/&/=a-zA-Z0-9]*" );

                this.showAlert(
                    statusMessage,
                    ( url != null && url.length != 0 ) ?
                        ( "OpenURL( 'http://" + url + "' );" ) :
                        "" );
            }
            else
            {
                this.ClearAlertWithTimeStamp( this.m_gettingCallsAlertMsgTimeStamp );
                this.ClearAlertWithTimeStamp( this.m_undeleteAlertMsgTimestamp );
            }

            this.refresh( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::OnCallListChanged( )' );
        }
    },

    OnCallListChangedAndActivated : function( statusMessage )
    {
        try
        {
            this.m_activationStatus = TYPE_ACTIVATION_STATE_ACTIVATED;

            this.OnCallListChanged( statusMessage );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::OnCallListChangedAndActivated( )' );
        }
    },

    OnNewCallsAvailable:function( )
    {
    },

    OnSyncFailed:function( errorMessage )
    {
        EX_Log( "VM_CallListView::OnSyncFailed : " + errorMessage  ? errorMessage : '' );
    },

    OnCallListSyncCompletedNoChange:function( statusMessage )
    {
        try
        {
            this.showAlert( statusMessage );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::OnCallListSyncCompletedNoChange( )' );
        }
    },

    /**************************************************************************
      Activation methods
     **************************************************************************/

    SendCarrierInfo:function( )
    {
        try
        {
            document.getElementById( 'WIDGET_activationStatusTextArea' ).innerHTML = '';

            this.m_activationHandler.setCarrierInfo( VM_GetPreference( PREF_KEY_ACCOUNT ),
                                                     VM_GetPreference( PREF_KEY_U_STRING ),
                                                     this.m_carrierId );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::SendCarrierInfo( )' );
        }
    },

    OnSetCarrierInfoSucceeded: function( responseMessage )
    {
        try
        {
            switch ( responseMessage.m_status )
            {
            case 0:
                {
                    this.ClearAlertWithTimeStamp( this.m_carrierInfoAlertMsgTimeStamp );

                    document.getElementById( 'WIDGET_activationStatusTextArea' ).innerHTML = "(Account updated)";
                    break;
                }

            case 11:
                {
                    this.m_onLoginRequired( responseMessage.m_statusMsg );
                    break;
                }

            case 2:
            default:
                {
                    var alertMessage = "Sorry, failed to update your account due to a server error. Please try again later.";

                    if ( responseMessage.m_statusMsg != null && responseMessage.m_statusMsg.length != 0 )
                    {
                        alertMessage = responseMessage.m_statusMsg;
                    }

                    this.m_carrierInfoAlertMsgTimeStamp = this.showAlert( alertMessage, "" );

                    break;
                }
            }
        }
        catch ( e )
        {
            EX_Log( "VM_CallListView::OnSetCarrierInfoSucceeded( ) : " + e.message ? e.message : UTILS.dump(e) );
        }
    },

    OnSetCarrierInfoFailed: function( statusMessage )
    {
        try
        {
            this.m_carrierInfoAlertMsgTimeStamp =
                this.showAlert(
                    "Sorry, failed to update your account due to a server error. Please try again later.", "" );

            EX_Log( "VM_CallListView::OnSetCarrierInfoFailed( ) : " + statusMessage );
        }
        catch ( e )
        {
            EX_Log( "VM_CallListView::OnSetCarrierInfoFailed( ) : " + e.message );
        }
    },


    /**************************************************************************
      Helper methods
     **************************************************************************/
    MoveElement : function( tagName, sourceParentElement, destinationParentElement )
    {
        try
        {
            // let the children out of the closet
            var childrenInTheCloset = sourceParentElement.childNodes;
            var locatedElement = null;

            var nodeNameString = 'Node names: ';
            for ( var i = 0; i < childrenInTheCloset.length; ++i )
            {
                var nodeName = childrenInTheCloset.item( i ).nodeName;

                nodeNameString += '"' + nodeName + '", ';

                if ( nodeName.toLowerCase( ) == tagName.toLowerCase( ) )
                {
                    locatedElement = childrenInTheCloset.item( i );
                    break;
                }
            }

            if ( locatedElement == null )
            {
                // die
                throw CWError( 'CantFindTag', 'Tag: ' + tagName + '\n' + nodeNameString );
            }

            destinationParentElement.appendChild( locatedElement );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::MoveElement( )' );
        }
    },

    shorten : function( value )
        {
            var len = value.length;

            // deal with the cases where only dots are left
            if (len <= 1)
            {
                // we'll always leave a dot, even if it's too wide
            return value;
            }

        if (len <= 3)
            {
                // once we're down to the ellipsis, just remove dots from the end
            return value.substr(0, len - 2);
            }

        if (value.charAt(len - 4) == ";")
        {
            // handle the case where the character before the ellipsis is escaped, e.g., &amp;
            value = value.replace(/&\S;\.{3}$/, "...");

            if ( value.length < len )
            {
                return value;
            }
        }

        return value.substr(0, len - 5) + "...";
    },

    // Used only by GetConfinedString
    // since we call this function multiple times per row in the call list
    // we should keep our hidden div around so we don't have to recreate it
    m_GCS_div: null,
    GetConfinedString : function( value, maxWidth )
    {
        try
        {
            if (!this.m_GCS_div)
            {
                // create a hidden div to use for measurement
                this.m_GCS_div = document.createElement("div");
                this.m_GCS_div.style.visibility = "hidden";
                this.m_GCS_div.style.position = "absolute";
                this.m_GCS_div.style.borderWidth = "0px";
                this.m_GCS_div.style.padding = "0px";
                this.m_GCS_div.style.font = "13px Helvetica";
                document.body.appendChild(this.m_GCS_div);
            }

            this.m_GCS_div.innerHTML = value;

            if ( this.m_GCS_div.clientWidth <= maxWidth || value.length <= 1 )
            {
                return value;
            }

            value = this.shorten( value + "..." );

            var spinCount = 0;

            for ( ; value.length > 1; ++spinCount )
            {
                this.m_GCS_div.innerHTML = value;

                if ( this.m_GCS_div.clientWidth <= maxWidth )
                {
                    break;
                }

                value = this.shorten( value );
            }


            //EX_Log( 'GetConfinedString( ) - Spun ' + spinCount + ' for maxWidth=' + maxWidth + ' value="' + value + '"' );

            return value;
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::GetConfinedString( )' );
        }
    },

    GetCallDisplayValues : function( call )
    {
        try
        {
            var displayValues = new Object( );

            displayValues.callerName  = call.getCallerName( );
            displayValues.callerId    = call.getFormattedCallerId( );
            displayValues.callLoc     = call.getLocation( );

            displayValues.date_time_tz = call.getFormattedDate() + " " + call.getFormattedTime() + " " + this.m_callListHandler.m_tz;

            return displayValues;
            }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::GetCallDisplayValues( )' );
        }
    },

    GetCallInfoPortion: function( call, callWidths )
    {
        try
        {
            var displayValues = this.GetCallDisplayValues( call );

            var toBeDisplayedCallerName = ( displayValues.callerName && displayValues.callerName.length ?
                                            displayValues.callerName :  displayValues.callerId );

            // confine the value to the available width
            // -- NOTE this should probably be protected and have some sort of sane default
            toBeDisplayedCallerName = this.GetConfinedString( toBeDisplayedCallerName, callWidths.name );

            // create the tool tip for the phone type
            var phoneTypeTipHTML = '';
            var phoneTypeIcon = call.getPhoneTypeIcon( undefined, 0 );
            if ( phoneTypeIcon.length != 0 )
            {
                phoneTypeTipHTML = stag( 'img', { src: phoneTypeIcon } );
            }

            var theInfo = tag('div', { 'class': 'type' }, phoneTypeTipHTML ) +
                          tag('div', { 'class': 'name' }, UTILS.escapeHTML( toBeDisplayedCallerName ) ) +
                          tag('div', { 'class': 'date' }, call.getFormattedDate( ) ) +
                          tag('div', { 'class': 'time' }, call.getFormattedTime( ) ) +
                          tag('div', { 'class': 'duration' }, call.getOutcome( ) );

            return theInfo;
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::GetCallInfoPortion( )' );
        }
    },

    GetCallEditPortion: function( call )
    {
        try
        {
            var theEdit = stag('input', { name: 'WIDGET_callListRowCallerNameTextArea', type: 'text', value: call.getCallerName().replace(/\"/g, '&quot;'),
                                          onKeyDown:  "VM_widgetViewController.getCallListView( ).OnCallerNameKeyInput( event );",
                                          onClick: 'VM_CallListView_static.phoneTypeJustSelected();' }) +
                          '&nbsp;' +
                          stag('input', {name: 'phoneType_val', type: 'hidden', value: call.getPhoneType() } ) +
                          tag('select', {name: 'WIDGET_callListRowCallerPhoneTypeTextArea',
                                         onChange: "$('input[@name=phoneType_val]', $(this).parent()).val(this.options[this.selectedIndex].value); VM_CallListView_static.phoneTypeJustSelected();",
                                         onClick: 'VM_CallListView_static.justReturnFalse' },
                              this.getPhoneTypeOptionHTML( call, CL_PHONE_TYPE_RESIDENTIAL_PHONE ) +
                              this.getPhoneTypeOptionHTML( call, CL_PHONE_TYPE_BUSINESS_PHONE ) +
                              this.getPhoneTypeOptionHTML( call, CL_PHONE_TYPE_MOBILE_PHONE ) +
                              this.getPhoneTypeOptionHTML( call, CL_PHONE_TYPE_UNKNOWN_PHONE )
                              ) +
                          '&nbsp;' +
                          stag('input', { type: 'button', value: 'ok', 'class': 'ok button',
                                          onClick: 'VM_CallListView_static.phoneTypeJustSelected(); VM_widgetViewController.getCallListView().OnSaveCallerEditChanges();' });
                return theEdit;
            }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::GetCallEditPortion( )' );
        }
    },

    callRowStripe: ' stripe',
    getCallRow: function( call, callWidths )
    {
        try
        {
            var r = '';

            var playImg = '';

            if ( call.getDurationSecs( ) )
            {
                playImg = stag('img', { src: VM_RESOURCE_PLAY_IMAGE, height: 21, width: 21 } );
            }

            r += tag('td', { 'class': 'play' + (playImg ? ' hasMsg' : ''), onClick: 'VM_CallListView_static.handlePlayClick(this);' }, playImg );

            var theInfo = this.GetCallInfoPortion( call, callWidths );

            var theEdit = this.GetCallEditPortion( call );

            var meatOfTheCallRow =
                        '<table cellspacing="0" cellpadding="0" border="0"><tr><td style="height:25px">' +

                              tag( 'div',
                                   { 'class': 'info', onClick: 'VM_CallListView_static.handleMsgAreaClick(this);' },
                                   theInfo ) +

                              tag( 'div',
                                   { 'class': 'editInfo', onClick: 'VM_CallListView_static.handleMsgAreaClick(this);' },
                                   theEdit ) +

                        '</td></tr></table>' +

                         tag('div', { 'class': 'extra' },
                             tag('div', { 'class': 'player' },  this.getPlayer( call, 1 )) +
                             tag('div', { 'class': 'moreInfo' }, call.getFormattedCallerId() ) );

            r += tag('td', { 'class': 'meat' }, meatOfTheCallRow);

            if (call.getFormattedCallerId() && call.getFormattedCallerId().length > 0) {

                r += tag(   'td',
                            { 'class': 'edit', onClick: 'VM_CallListView_static.handleEditClick(this);' },
                            tag(    'img',
                                    { src: VM_RESOURCE_EDIT_IMAGE, height: 21, width: 21 },
                                    '',
                                    1 ) );
            }
            else
            {
                // not callerID information available so you can't edit this call - cr15054
                //  - usually 'private number' or 'unavailable' calls.
                r += tag('td', { 'class': 'edit notAllowed', onClick: 'VM_CallListView_static.handleMsgAreaClick(this);' }, "&nbsp;" );
            }

            r += tag('td', { 'class': 'delete', onClick: 'VM_CallListView_static.handleDeleteClick(this);' },
                         tag('img', { src: VM_RESOURCE_DELETE_IMAGE, height: 21, width: 21 }, '', 1 ) );


            this.callRowStripe = this.callRowStripe != ' stripe' ? ' stripe' : '';

            var vTxt = call.getNormalizedVtxt( );

            if ( vTxt )
            {
                vTxt = 'header=[&nbsp;Vtxt<sup>SM</sup> GIST] body=[' + vTxt + '] delay=[' + this.VTEXT_DISPLAY_DELAY_MS + ']';
            }

            return tag( 'tr',
                        {
                            'id': call.getCallId(), onDblClick: 'VM_CallListView_static.handlePlayClick(this);',
                            'title': vTxt,
                            'class': 'call' + this.callRowStripe +
                               (call.getReviewStatus() == CL_REVIEW_STATE_NEW ? ' new' : '') +
                               (call.selected ? ' hot' : '')
                        },
                       r );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::getCallRow( )' );
        }

    },

    SetAndDisplayWirelessProviderInfo : function( )
    {
        try
        {
            if ( this.m_carrierList            == null ||
                 this.m_carrierList.m_carriers == null )
            {
                return;
            }

            var carrierList       = this.m_carrierList.m_carriers;
            var selectedCarrierId = this.m_carrierId;

            var forwardCodeId     = -1;
            var carrierName       = '';

            if ( selectedCarrierId != VM_CARRIER_ID_OTHER )
            {
                for ( var i = 0; i < carrierList.length; ++i )
                {
                    if ( carrierList[i].m_id == selectedCarrierId )
                    {
                        var carrier = carrierList[i];

                        forwardCodeId = carrier.m_fwdCodeId;
                        carrierName   = carrier.m_name;

                        break;
                    }
                }
            }

            this.DisplayActivationCodes( selectedCarrierId, forwardCodeId, carrierName, this.m_hideRegCarrier === 0 );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::SetAndDisplayWirelessProviderInfo( )' );
        }
    },

    DisplayActivationCodes : function( carrierId, forwardCodeId, carrierName, showCarrierSelection )
    {
        try
        {
            var step1Area               = document.getElementById( 'WIDGET_activationStep1TextArea' );
            var selectCarrierPopup      = document.getElementById( 'WIDGET_wirelessProviderPopup' );
            var step2Area               = document.getElementById( 'WIDGET_activationStep2TextArea' );
            var callToActionArea        = document.getElementById( 'WIDGET_callToActionTextArea' );
            var activationCodesArea     = document.getElementById( 'WIDGET_activationActCodesTextArea' );
            var step3Area               = document.getElementById( 'WIDGET_activationStep3TextArea' );
            var deactivationArea        = document.getElementById( 'WIDGET_activationDeactivationTextArea' );
            var deactivationCodesArea   = document.getElementById( 'WIDGET_activationDeactCodesTextArea' );

            step1Area.style.display             = 'none';
            selectCarrierPopup.style.display    = 'none';
            step2Area.style.display             = 'none';
            callToActionArea.style.display      = 'none';
            activationCodesArea.style.display   = 'none';
            step3Area.style.display             = 'none';
            deactivationArea.style.display      = 'none';
            deactivationCodesArea.style.display = 'none';

            activationCodesArea.innerHTML   = '';
            deactivationCodesArea.innerHTML = '';

            var stepNumber = 1;

            if ( showCarrierSelection )
            {
                step1Area.innerHTML = (stepNumber++) + ". Select your cell phone carrier:";
                step1Area.style.display = 'inline';
                selectCarrierPopup.style.display = 'inline';
            }

            if ( carrierId == VM_CARRIER_ID_OTHER || forwardCodeId == 0 )
            {
                step2Area.innerHTML = stepNumber + ". We do not have specific instructions for your carrier, but activation may be available.";
                step2Area.style.display = 'block';

                callToActionArea.innerHTML = '<a href="#" onclick="OpenURL( \'' + VM_URL.WWW + '/v2w/faq.html' + VM_getDefaultUrlQueryParameters( ) + '\' );">Please check our FAQ for a complete list.</a>';
                callToActionArea.style.display = 'block';

                return;
            }

            if ( forwardCodeId == -1 )
            {
                var displayedCarrierName = "your carrier";

                if ( carrierName != null && carrierName.length != 0 && carrierId != VM_CARRIER_ID_OTHER )
                {
                    displayedCarrierName = carrierName;
                }

                deactivationArea.innerHTML = "";
                step3Area.innerHTML = "";

                step2Area.innerHTML =
                    stepNumber + ". CallWave works with most major carriers including Cingular, Verizon, " +
                    "and T-Mobile. Unfortunately activation is not available for " + displayedCarrierName + "."

                step2Area.style.display = 'block';


                callToActionArea.innerHTML = '<a href="#" onclick="OpenURL( \'' + VM_URL.WWW + '/v2w/otherWidgets.html' + VM_getDefaultUrlQueryParameters( ) + '\' );">Check out CallWave\'s other widgets & gadgets.</a>';
                callToActionArea.style.display = 'block';

                return;
            }

            var forwardCodeList   = this.m_forwardCodeList.m_forwardCodeList;
            var activationCodes   = "";
            var deactivationCodes = "";

            for ( var i = 0; i < forwardCodeList.length; ++i )
            {
                if ( forwardCodeList[i].m_id == forwardCodeId )
                {
                    activationCodes   = forwardCodeList[i].m_actStr;
                    deactivationCodes = forwardCodeList[i].m_deactStr;

                    break;
                }
            }

            step2Area.innerHTML =
                (stepNumber++) + ". Activate by dialing " + ( this.GetLineCount( activationCodes ) > 1 ? "these codes" : "this code" ) +
                " on your cell phone and then press SEND.";

            activationCodesArea.innerHTML = activationCodes;

            step3Area.innerHTML = (stepNumber++) + ". Try it out! Call your cell phone and leave a message. You'll see it right here.";

            // align to the right
            var callToActionAreaHTML =
                '<table width="100%" cellspacing="0" cellpadding="0">'  +
                '   <tr><td align="right">'                             +
                '       <a href="#" onclick="OpenURL( \'' + VM_URL.WWW + '/v2w/notWorking.html' + VM_getDefaultUrlQueryParameters( ) + '\' );">Didn\'t work?</a>' +
                '   </td></tr>'                                         +
                '</table>';

            callToActionArea.innerHTML = callToActionAreaHTML;

            deactivationArea.innerHTML = "Should you ever need to deactivate, use this code:";

            deactivationCodesArea.innerHTML = deactivationCodes;

            step2Area.style.display             = 'inline';
            callToActionArea.style.display      = 'inline';
            activationCodesArea.style.display   = 'inline';
            step3Area.style.display             = 'inline';
            deactivationArea.style.display      = 'inline';
            deactivationCodesArea.style.display = 'inline';
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_CallListView::DisplayActivationCodes( )' );
        }
    },

    GetLineCount : function( stringValue )
    {
        var lineCount = 1;

        for ( var i = 0; i < stringValue.length; ++i )
        {
            if ( stringValue.charAt( i ) == '\n' )
            {
                ++lineCount;
            }
        }

        return lineCount;
    },


    /**************************************************************************
      Call List GUI event handlers
     **************************************************************************/
    OnCallListRowSelected: function( rowIndex, timeStamp, callInstance, messageDurationSecs )
    {
        try
        {
            if (arguments.length == 1) {
                var callId = rowIndex; // single arguments pass a callId
                //alert( "OnCallListRowSelected: "  + (new Date()).toString() + "; " + (callId ? callId : typeof callId + " :(") );
                if (this.m_selectedCallId && this.m_selectedCallId != callId) {
                    //alert( "OnCallListRowSelected: previously " + this.m_selectedCallId + " selected: " + this.m_callList.at( this.m_selectedCallId ).selected);
                    this.m_callList.at( this.m_selectedCallId ).selected = ''; // clear out selected on old selected call
                }
                this.m_selectedCallId = callId;

                var call = this.m_callList.at( this.m_selectedCallId );
                call.selected = ' selected';
                //alert( "OnCallListRowSelected: " + this.m_selectedCallId + (new Date()).toString());
                if (call.getDurationSecs() == 0) {
                    $("#" + this.m_selectedCallId).filter(".new").removeClass("new");
                    this.m_callListHandler.handleCallSaved( this.m_selectedCallId );
                }
                //this.refresh(); ??? not sure this is needed.
                //alert( "post handleCallSaved " + (new Date()).toString() );
                if (window._IG_AdjustIFrameHeight) window._IG_AdjustIFrameHeight();
                return;
            }

        }
        catch ( e )
        {
            EX_Log( "VM_CallListView::OnCallListRowSelected( ) : " + e.message );
        }
    },

    ShowCallPortion : function( portion, call )
    {
        $( portion, call ).show( );
    },

    HideCallPortion : function( portion, call )
    {
        $( portion, call ).hide( );
    },

    ShowInfoHideEditPortions : function( call )
    {
        this.HideCallPortion( ".info", call );
        this.HideCallPortion( ".editInfo", call );

        this.ShowCallPortion( ".info", call );
    },

    ShowEditHideInfoPortions : function( call )
    {
        this.HideCallPortion( ".info", call );
        this.HideCallPortion( ".editInfo", call );

        this.ShowCallPortion( ".editInfo", call );
    },

    OnEditCaller : function( callId )
    {
        try
        {
            VM_CallListView_static.DisableVtxt( );

            this.m_editingCallId = CL_CallId.fromString( callId );

            var call = $("#" + callId);

            this.ShowEditHideInfoPortions( call );

            var nameArea = $(".editInfo input[@name=WIDGET_callListRowCallerNameTextArea]", call)[0]; // get to the DOM element, yes this [0] stuff is a bit ugly.
            nameArea.focus();
            nameArea.select(0,-1);
        }
        catch ( e )
        {
            EX_Log( "VM_CallListView::OnEditCaller( ) : " + e.message );
        }
    },

    OnSaveCallerEditChanges: function( _callId )
    {
        try
        {
            var callId = _callId ? _callId : this.m_editingCallId;
            if (!callId) {
                EX_Log( "OnSaveCallerEditChanges: Attempting to save w/o a specific call in mind." );
                return;
            }
            var call = $("#" + callId);

            // Update caller name
            var newCallerName = $('input[@name=WIDGET_callListRowCallerNameTextArea]', call).val()
            var namesUpdated = this.m_callListHandler.updateCallerName( this.m_editingCallId, newCallerName );

            // Update caller phone type
            var selectedPhoneType = $('input[@name=phoneType_val]', call).val();
            selectedPhoneType ? selectedPhoneType : '';
            var phoneTypesUpdated = this.m_callListHandler.updatePhoneType( this.m_editingCallId, selectedPhoneType );

            var callsUpdated = {}; // merge the namesUpdated and phoneTypesUpdated lists
            for (var i=0; i < phoneTypesUpdated.length; i++) {
                callsUpdated[phoneTypesUpdated[i]] = 1;
            }
            for (var i=0; i < namesUpdated.length; i++) {
                callsUpdated[namesUpdated[i]] = 1;
            }

            // update call row (just updated parts of the meat section)
            // - Maybe we should do the  $(call).html() or replace the call element all together?
            //   -- NO.  you loose the player state by replacing the entire call element.
            // - Keeping the element there keeps some display state stuff in the html for us.
            var callWidths = this.GetCallWidths( );
            for (callId in callsUpdated) {
                call = $("#" + callId);
                if (call) {
                    $(".meat .editInfo", call).html( this.GetCallEditPortion( this.m_callList.at( callId ) ) );
                    $(".meat .info", call).html( this.GetCallInfoPortion( this.m_callList.at( callId ), callWidths ) );
                }
            }

            this.cancelEditing( );
        }
        catch ( e )
        {
            EX_Log( "VM_CallListView::OnSaveCallerEditChanges( ) : " + e.message );
        }
    },

    OnDeleteCall: function( timeStamp, callInstance )
    {
        try
        {
            if (arguments.length == 1)
            {
                // called with just the dom object
                var call = $(timeStamp).parent();

                this.m_callListHandler.handleCallDeleted( call.id() );

                if (call.id() == this.m_selectedCallId) {
                    this.m_selectedCallId = null; // clear out the currently selected call if it is deleted
                }

                this.m_undeleteAlertMsgTimestamp =
                    this.showAlert( "Call deleted - Click here to undo.",
                                    "VM_widgetViewController.getCallListView( ).OnUndeleteCall( '" + call.id() + "' );" );

                this.refresh();

                return;
            }
        }
        catch ( e )
        {
            EX_Log( "VM_CallListView::OnDeleteCall( ) : " + e.message ? e.message : UTILS.dump(e));
        }
    },

    OnUndeleteCall: function( timeStamp, callInstance )
    {
        try
        {
            if (arguments.length == 1) {
                var callId = timeStamp;
                this.m_callListHandler.handleCallUndeleted( callId );
                var call = VM_widgetViewController.getCallListView().m_callList.at( callId );
                if (call && call.getReviewStatus() == CL_REVIEW_STATE_NEW) {
                    this.showAlert( "Call undeleted.", 5 ); // TODO - add timeout to clear this msg.
                }
                else {
                    this.showAlert( "Unable to undelete call.", 5 );
                }

                this.refresh();

                return;
            }

        }
        catch ( e )
        {
            EX_Log( "VM_CallListView::OnUndeleteCall( ) : " + e.message ? e.message : UTILS.dump(e) );
        }
    },

    OnCallerNameKeyInput : function( event )
    {
        try
        {
            if ( ! this.isEditing( ) )
            {
                return;
            }

            // get the key code
            var keyCode = null;
            if ( window.event )
            {
                keyCode = event.keyCode;
            }
            else if ( event.which )
            {
                keyCode = event.which;
            }

            if ( keyCode == null )
            {
                return;
            }

            var KEY_CODE_ENTER  = 13;
            var KEY_CODE_ESCAPE = 27;

            switch ( keyCode )
            {
            case KEY_CODE_ENTER:
                {
                    this.OnSaveCallerEditChanges( );
                }
                break;

            case KEY_CODE_ESCAPE:
                {
                    // Changes discarded
                    this.cancelEditing( );
                    this.refresh( );
                }
                break;
            }
        }
        catch ( e )
        {
            EX_Log( "VM_CallListView::OnCallerNameKeyInput( ) : " + e.message );
        }
    },

    getPlayer: function(call, stopped) {
        if (call && call.getDurationSecs && call.getDurationSecs() == 0) {
            return ''; // return no player if there is no msg
        }

        if (stopped) {
            return stag('img', VM_RESOURCE_PLAYER_STUB_IMG);
        }
        var FLASH = {
            classid: 'clsid:d27cdb6e-ae6d-11cf-96b8-444553540000',
            codebase: 'http://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=8,0,0,0',
            movie: VM_URL.WWW + '/members/mediaPlayerWidget.swf',
            pluginspage: 'http://www.macromedia.com/go/getflashplayer',
            height: 26,
            width: 142
        };

        // the message fetch url must be escaped for the request to succeed
        var queryString = escape(call.getMessagePrimaryFetch( ) + '*' +
                                 call.getMessageSecondaryFetch( ) +
                                 '&os=' + navigator.appVersion +
                                 '&b=' + navigator.appName );

        queryString = '?color=0xbf0000' +
            '&autoPlay=true' +
            '&avFile=' + queryString +
            '&md=' + (call.getDurationSecs( ) * 1000); // [m]essage [d]uration is in ms

        var objParams = {
            classid: FLASH.classid,
            codebase: FLASH.codebase,
            width: FLASH.width, height: FLASH.height,
            id: 'mediaPlayer_HT2'
        };
        var movieParams = {
            classid: FLASH.classid,
            codebase: FLASH.codebase,
            width: FLASH.width, height: FLASH.height,
            src: FLASH.movie + queryString,
            quality: 'high',
            name: objParams.id,
            allowScriptAccess: 'sameDomain',
            type: 'application/x-shockwave-flash',
            pluginspage: FLASH.pluginspage,
            bgcolor: '#b5d6e7'
        }


        return tag('div', {'class':'player'},
                   tag('object', objParams,
                       stag('param', {name: 'allowScriptAccess', value: movieParams.allowScriptAccess}) +
                       stag('param', {name: 'movie'            , value: movieParams.src}) +
                       stag('param', {name: 'quality'          , value: movieParams.quality}) +
                       stag('param', {name: 'bgcolor'          , value: movieParams.bgcolor}) +
                       stag('embed', movieParams)
                       )
                   );
    },

    getPhoneTypeOptionHTML: function(call, phoneType)
    {
        return tag('option', {value: phoneType, selected: phoneType == call.getPhoneType( ) ? 'selected' : undefined},
                    CL_GetPhoneTypeAsString( phoneType ) );
    }

};

VM_CallListView_static = {

    setupSingleCall: function(){
        // expects this to be the DOM object for the call you want to setup.
        var call = $(this);
        //alert( "setupSingleCall ... " + call.id() );

        if ($(call).is('.hot')) {
            VM_CallListView_static.handleCallClick( call ); // show the call
        }
        else{
            $(".extra:visible", call).hide(); // make sure the extra bits are not visible
        }
    },

    playTimeout: null,

    handleCallClick: function(clickedCall, playClicked){
       //alert( "handleCallClick called (" + typeof clickedCall + ", " + playClicked +")" );
       var calls = $("#calls .call");
       $(".extra:visible", $(calls).not(clickedCall)).hide();

        // stop everyone else
       $(calls).filter(".playing").each( function() {
           if ($(this).id() != $(clickedCall).id()) {
               VM_CallListView_static.stopPlayerForCall.call(this);
           }
       });

       $(".extra:hidden", clickedCall).show(); // .extra holds the 'player' and the callerID

       $(clickedCall).addClass("hot")
            .siblings().filter(".hot").removeClass("hot");

        if (playClicked)
        {
            //alert( "playClicked for clickedCall.id: " + $(clickedCall).id() );
            //alert( "is .playing: " + $(clickedCall).is(".playing"));

            if ($(clickedCall).is('.playing'))
            {
                VM_CallListView_static.playTimeout && clearTimeout( VM_CallListView_static.playTimeout );
                $(clickedCall).removeClass("playing").each( VM_CallListView_static.stopPlayerForCall );
            }
            else
            {
                VM_CallListView_static.DisableVtxt( );
                var call;
                try
                {
                    call = VM_widgetViewController.getCallListView().m_callList.at( $(clickedCall).id() );
                }
                catch(e)
                {
                    EX_Log( "VM_CallListView_static::handleCallClick(): error fetching CL_Call for call clicked. " + UTILS.dump(e) );
                }

                VM_CallListView_static.unbindAll( $(".player", clickedCall) );

                $(".player", clickedCall).html( VM_widgetViewController.getCallListView().getPlayer( call ) );
                $(".play img", clickedCall).src( VM_RESOURCE_STOP_IMAGE );
                $(clickedCall).addClass("playing");

                var callIdToStop = $(clickedCall).id();
                var stopIn = call.getDurationSecs();
                if (stopIn < 10 ) {
                    stopIn = 1;
                }
                else if (stopIn < 30) {
                    stopIn = stopIn * 0.1;
                }
                else {
                    stopIn = 3;
                }
                stopIn = (call.getDurationSecs() + stopIn) * 1000;

                VM_CallListView_static.playTimeout && clearTimeout( VM_CallListView_static.playTimeout );
                VM_CallListView_static.playTimeout = setTimeout( function() {
                    var call = VM_widgetViewController.getCallListView().m_callList.at( callIdToStop );
                    if (call) {
                        //alert( "auto-magic guessing STOP ... " + VM_CallListView_static.playTimeout + "; " + call.getCallId() + ", " + call.getDurationSecs());
                        VM_CallListView_static.playTimeout = null;
                        var clickedCall = $("#" + callIdToStop);
                        $(clickedCall).removeClass("playing").each( VM_CallListView_static.stopPlayerForCall );
                    } }, stopIn);
                //alert( call.getCallId() + " .. should stop in " + stopIn);

                // Change the review state of the call
                $(clickedCall).removeClass("new"); // the visual/view update
                VM_widgetViewController.getCallListView().m_callListHandler.handleCallSaved( $(clickedCall).id() ); // and update the call itself
            }
        }

        VM_widgetViewController.getCallListView().OnCallListRowSelected( $(clickedCall).id() ) ;

        return false;
    },

    stopPlayerForAllCalls: function() {
        VM_CallListView_static.playTimeout && clearTimeout( VM_CallListView_static.playTimeout );
        $(".playing", "#calls").each( VM_CallListView_static.stopPlayerForCall );
    },

    stopPlayerForCall: function()
    {
        VM_CallListView_static.EnableVtxt( );

        //alert( "stopPlayerForCall " + $(this).attr("id");
        // TODO - maybe add conditional to kick out if not .playing?

        $(this).removeClass("playing");
        VM_CallListView_static.unbindAll( $(".player", this) );
        $(".player", this).html( VM_widgetViewController.getCallListView().getPlayer( null, 1 ) );
        $(".play img", this).src( VM_RESOURCE_PLAY_IMAGE );
    },

    stripeIt: function(){ var i = 0; $("#calls .call:visible").each(function(){ (++i % 2) ? $(this).addClass('stripe') : $(this).removeClass('stripe'); }); },

    unbindAll: function( node )
    {
        try
        {
            node = typeof node != 'number' && node ? node : this;

            if (typeof node == 'function' && node.nodeName == 'OBJECT')
            {
                return; // don't try to unbind anything from an OBJECT tag.  (usually a flash object)
            }

            $(node).unbind().children().each( VM_CallListView_static.unbindAll );
        }
        catch (e)
        {
            EX_Log( "VM_CallListView_static::unbindAll EX: " + typeof node + "\n" + UTILS.dump(e) );
        }
    },

    handlePlayClick: function( clickedCallDOMElement )
    {
        var clickedCall = VM_CallListView_static.GetJCallFromDOMElement( clickedCallDOMElement );

        VM_CallListView_static.handleCallClick( clickedCall, $(".play", clickedCall).is('.hasMsg') );
    },

    timeoutPhoneTypeJustSelected: null, // used in the 'edit' form input elements onClick handlers to suppress the click event since I can't seem to disable the bubbling of the click event.
    phoneTypeJustSelected: function() {
        VM_CallListView_static.timeoutPhoneTypeJustSelected = setTimeout( function() { VM_CallListView_static.timeoutPhoneTypeJustSelected = null; }, 10);
    },

    ResetVtxt : function ( )
    {
        BoxOver.ResetDisplayState( );
    },

    EnableVtxt : function( )
    {
        BoxOver.Enable( );
    },

    DisableVtxt : function( )
    {
        BoxOver.Disable( );
    },

    handleMsgAreaClick: function( clickedCallDOMElement )
    {
        if (VM_CallListView_static.timeoutPhoneTypeJustSelected)
            return;

        var clickedCall = VM_CallListView_static.GetJCallFromDOMElement( clickedCallDOMElement );

        VM_CallListView_static.handleCallClick( clickedCall );
    },

    GetJCallFromDOMElement : function( callDOMElement )
    {
        var clickedCall = $( callDOMElement );
        for (;clickedCall.id() == ''; clickedCall = clickedCall.parent()) {} // find callid for this click
        return clickedCall;
    },

    handleEditClick: function( clickedCallDOMElement )
    {
        try
        {
            var clickedCall = VM_CallListView_static.GetJCallFromDOMElement( clickedCallDOMElement );

            var editingCallId = VM_widgetViewController.getCallListView( ).m_editingCallId;

            VM_widgetViewController.getCallListView( ).cancelEditing( );

            if ( editingCallId === null || ! editingCallId.equals( CL_CallId.fromString( clickedCall.id( ) ) ) )
            {
                VM_widgetViewController.getCallListView().OnEditCaller( clickedCall.id() );
            }
        }
        catch ( e )
        {
            EX_Log( "VM_CallListView_static::handleEditClick EX: \n" + e.message );
        }
    },

    handleDeleteClick: function( call )
    {
        //alert( "delete clicked");
        VM_widgetViewController.getCallListView().OnDeleteCall(call);
    },

    justReturnFalse: function() { return false; }


}
